local Module = LibStub("AceAddon-3.0"):NewAddon("LH_Drops")

local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LibStub("LibDBIcon-1.0", true)

function Module:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("LH_DropsDB", {
        profile = {
            minimapIcon = {},
            hideMinimapOption = nil,
            loottable = 3
        }
    })

    local LDBObj = LDB:NewDataObject("Little Helper: Mob Value",  {
        type = "data source",
        icon = [[Interface\Icons\inv_chest_cloth_18]],
        OnClick = function () self:ShowMenu() end,
        OnTooltipShow = function (tooltip) self:ShowTooltip(tooltip) end})

    LDBIcon:Register("Mob Value", LDBObj, self.db.profile.minimapIcon)

    hooksecurefunc (GameTooltip, "SetUnit",
        function(tooltip, unit)
            self:OnUpdate ()
        end
    );

    local frame = CreateFrame("Frame", nil)
    frame:SetScript("OnUpdate", function () self:OnUpdate() end)
    self.lastUnit = nil
end

function Module:Val2gsc (v)
    local rv = math.floor (v + 0.5)

    local g = math.floor (rv / 10000)
    rv = rv - g * 10000

    local s = math.floor (rv / 100)
    rv = rv - s * 100

    local c = rv

    return g, s, c
  end

function Module:PriceToMoneyString (val)
    local gold, silver, copper  = self:Val2gsc(val);
  
    local st = "";
  
    if (gold ~= 0) then
      st = gold .. "|TInterface\\MoneyFrame\\UI-GoldIcon:12:12:4:0|t" .. "  ";
    end

    if (st ~= "") then
      st = st .. format("%02i%s  ",silver , "|TInterface\\MoneyFrame\\UI-SilverIcon:12:12:4:0|t");
    elseif (silver ~= 0) then
      st = st .. silver .. "|TInterface\\MoneyFrame\\UI-SilverIcon:12:12:4:0|t" .. "  ";
    end

    if (copper == 0) then
      return st;
    end

    if (st ~= "") then
      st = st .. format("%02i%s",copper , "|TInterface\\MoneyFrame\\UI-CopperIcon:12:12:4:0|t");
    elseif (copper ~= 0) then
      st = st .. copper .. "|TInterface\\MoneyFrame\\UI-CopperIcon:12:12:4:0|t";
    end

    return st;
end

function Module:OnUpdate ()
    local currentUnit = UnitGUID("mouseover")

    if currentUnit ~= self.lastUnit and currentUnit ~= nil then
        local type, a, b, c, d, e, f = string.split("-", currentUnit)

        local info = LH_DropsInfo[tonumber(e)]
        if info ~= nil then
           
            local lootTable = {}

            local value = 0
            for key, chance in pairs(info) do
                local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemIcon, itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(key)
                local vendorPrice, auctionPrice, dePrice = Atr_STWP_GetPrices (itemLink, 1, false, itemSellPrice, itemName, itemClassID, itemRarity, 1);

                local itemValue = 0
                if auctionPrice ~= nil then
                    itemValue = auctionPrice
                elseif vendorPrice ~= nil then
                    itemValue = vendorPrice
                end

                value = value + itemValue * chance

                table.insert(lootTable, {link=itemLink, chance=chance, value=itemValue})
            end

            if self.db.profile.loottable ~= 0 then
                table.sort(lootTable, function(a,b) return a.value > b.value end)

                GameTooltip:AddLine("\nLoot Table")

                for _,loot in  pairs(lootTable) do
                    GameTooltip:AddDoubleLine(loot.link, " (" ..  string.format ("%.2f",loot.chance*100) .. "%) " .. self:PriceToMoneyString(loot.value), 255,255,255, 255,255,0)
                    if _ == self.db.profile.loottable then
                        break
                    end
                end
            end

            GameTooltip:AddDoubleLine("Market Value", self:PriceToMoneyString(value), 255,00,0, 255,255,0)
            GameTooltip:Show()
        end
    end

    self.lastUnit = currentUnit
end

function Module:ShowMenu ()
    self.menuFrame = self.menuFrame or CreateFrame("Frame", "LH_Drops", UIParent, "UIDropDownMenuTemplate")
    local menuTable = {}

    menuTable[#menuTable+1] = { text = "Options",  notCheckable = true, isTitle = true }

    table.insert( menuTable, {
        func = function () self.db.profile.loottable = 0 end,
        text = "Show no loot table", 
        checked = self.db.profile.loottable == 0
    })

    table.insert( menuTable, {
        func = function () self.db.profile.loottable = 3 end,
        text = "Show 3 items from loot table", 
        checked = self.db.profile.loottable == 3
    })

    table.insert( menuTable, {
        func = function () self.db.profile.loottable = 5 end,
        text = "Show 5 items from loot table", 
        checked = self.db.profile.loottable == 5
    })

    table.insert( menuTable, {
        func = function () self.db.profile.loottable = 12 end,
        text = "Show 12 items from loot table", 
        checked = self.db.profile.loottable == 12
    })

    table.insert( menuTable, {
        func = function () self.db.profile.loottable = 800 end,
        text = "Show all items from loot table", 
        checked = self.db.profile.loottable == 800
    })

    if self.db.profile.hideMinimapOption ~= true then
        menuTable[#menuTable+1] = { text = "Minimap Icon",  notCheckable = true, isTitle = true }

        table.insert( menuTable, {
           func = function () 
                LDBIcon:Hide("Mob Value")
                self.db.profile.minimapIcon.hide = not self.db.profile.minimapIcon.hide
                if self.db.profile.minimapIcon.hide then
                    LDBIcon:Hide("Mob Value")
                else
                    LDBIcon:Show("Mob Value")
                end
            end,
           text = "Hide Minimap Icon", 
           checked = self.db.profile.minimapIcon.hide == true
       })
   
       table.insert( menuTable, {
           func = function () self.db.profile.hideMinimapOption = true end,
           text = "Hide 'Minimap Icon' Options", 
           checked = self.db.profile.hideMinimapOption == true
       })
     end

     EasyMenu(menuTable, self.menuFrame, "cursor", 0 , 0, "MENU", 1)
end

function Module:ShowTooltip (tooltip)

    tooltip:AddLine("Little Helper: Mob Value")
end
